
##############################################################################
# Description
# El angulo C vara entre -99999 y 99999 de forma continua
# Hace linearizacion cuando el cambio de angulos B o C es grande
# El caso de que la hta venga desde la vertical o vaya a la vertical est tratado en la linearizacion
# LA TOLERANCIA  de linearizacion es tollin y est en grados.
# Si la operacin empieza por G0B0C0 entonces se resetean los angulos y sale G0B0C0
################################################################################
#The following command invokes the debugging mode.
#_______________________________________________________________________________


#********Setting the debug mode ON/OFF
#  source [MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_review.tcl;  MOM_set_debug_mode ON

#______________________________________________________________________________
#The following command allows to access the machine tool simulation functions
#______________________________________________________________________________
source [MOM_ask_env_var UGII_CAM_POST_DIR]dmu_tt_sim_vnc.tcl


####### KINEMATIC VARIABLE DECLARATIONS ##############
  set mom_kin_min_arc_radius			.5
  set mom_kin_max_arc_radius			5000
  set mom_kin_arc_output_mode			QUADRANT
  set mom_kin_arc_valid_plane                    XY

#  set mom_kin_linearization_flag                "1"  
#  set mom_kin_linearization_tol                 "0.001"
  set mom_kin_machine_resolution                ".0001"
  set mom_kin_machine_type                      "3_axis_mill"



#_______________________________________________________________________________
#The following file consists of default values set for G & M codes
#_______________________________________________________________________________

#********M code declaration

         set mom_sys_program_stop_code              0
         set mom_sys_optional_stop_code             1
         set mom_sys_end_of_program_code            30

         set mom_sys_spindle_direction_code(CLW)    3
         set mom_sys_spindle_direction_code(CCLW)   4
         set mom_sys_spindle_direction_code(OFF)    5

         set mom_sys_tool_change_code               6

         set mom_sys_coolant_code(MIST)             7
         set mom_sys_coolant_code(FLOOD)            8
         set mom_sys_coolant_code(TAP)              8
         set mom_sys_coolant_code(OFF)              9


         set mom_sys_clamp_code(ON)                 10
         set mom_sys_clamp_code(OFF)                11
         set mom_sys_clamp_code(AXISON)             10
         set mom_sys_clamp_code(AXISOFF)            11

#********G code declaration

         set mom_sys_rapid_code                     0

         set mom_sys_linear_code                    1

         set mom_sys_circle_code(CLW)               2
         set mom_sys_circle_code(CCLW)              3

         set mom_sys_delay_code(REVOLUTIONS)        4
         set mom_sys_delay_code(SECONDS)            4

         set mom_sys_cutcom_plane_code(XY)          17
         set mom_sys_cutcom_plane_code(ZX)          18
         set mom_sys_cutcom_plane_code(YZ)          19

         set mom_sys_cutcom_code(OFF)               40
         set mom_sys_cutcom_code(LEFT)              41
         set mom_sys_cutcom_code(RIGHT)             42

         set mom_sys_inch_code                      70
         set mom_sys_metric_code                    71

         set mom_sys_cycle_breakchip_code           73
         set mom_sys_cycle_bore_no_drag_code        76

         set mom_sys_cycle_off                      80
         set mom_sys_cycle_drill_code               81
         set mom_sys_cycle_drill_dwell_code         82
         set mom_sys_cycle_drill_deep_code          83
         set mom_sys_cycle_tap_code                 84
         set mom_sys_cycle_bore_code                85
         set mom_sys_cycle_bore_drag_code           86
         set mom_sys_cycle_bore_back_code           87
         set mom_sys_cycle_bore_manual_code         88
         set mom_sys_cycle_bore_manual_dwell_code   89
         set mom_sys_cycle_bore_dwell_code          89
 
         set mom_sys_output_code(ABSOLUTE)          90
         set mom_sys_output_code(INCREMENTAL)       91

#_______________________________________________________________________________
# Global Variable Declaration
#_______________________________________________________________________________
         set mom_sys_list_file_columns              80
         set mom_sys_list_file_rows                 20
         set mom_sys_commentary_output              ON
	 set pi					    [expr 4*atan(1)]
	 set B					    0
	 set C					    0
	 set Bl					    0
	 set Cl					    0
	 set ang_B_old				    999
	 set ang_C_old				    999
	 set alfa				    0
	 set beta				    0
	 set tiempo_espera			    0
	 set xi					    0
	 set yi					    0
	 set zi					    0
	 set xc					    0
	 set yc					    0
	 set zc					    0
	 set xfc				    0
	 set yfc				    0
	 set zfc				    0
	 set xf					    0
	 set yf					    0
	 set zf					    0
	 set xfl				    0
	 set yfl				    0
	 set zfl				    0
	 set i					    0
	 set j					    0
	 set A					    0
	 set tipo_angulo			    0
	 set Cabs				    0
	 set Cant				    0
	 set Cinc				    0
	 set Bant				    0
	 set Binc				    0
	 set num_mov				    0 
	 set num_pasos_lin			    0
	 set xfsave				    0
	 set yfsave				    0
	 set zfsave				    0
	 set Bsave				    0
	 set Csave				    0
	 set xlinc				    0
	 set ylinc				    0
	 set zlinc			            0
	 set ilinc				    0
	 set jlinc				    0
	 set klinc				    0
	 set contador				    0
	 set Cabs_antiguo			    0
	 set Cfinal				    0
# tolerancia de linearizacion en grados
	 set tollin				    2
#para interpolacion circular
	 set ccw_cw				    0
#por cambios en el avance con el giro C
	set avance				    0
			
#traido de ugpost_base.tcl
proc  EQ_is_gt {s t} {
         global mom_system_tolerance

         if { $s > [expr ($t + $mom_system_tolerance)] } { return 1 } else { return 0 }
}




proc linearizacion {} {
#_______________________________________________________________________________
# calcula puntos intermedios cuando la rotacin es grande (segn tollin)
#
#_______________________________________________________________________________
global xfl yfl zfl Bl Cl
global xi yi zi
global num_pasos_lin
global mom_prev_mcs_goto mom_prev_tool_axis
global mom_mcs_goto mom_tool_axis 
global C Cinc Cabs inc1 Cant B Bant Cabs_antiguo Bsave Csave
global xlinc ylinc zlinc ilinc jlinc klinc
global contador tollin pi
global inc2 Bbase
global componente_k


set contador 0

#busco incrementos en x y z i j k
	set xlinc [expr ($mom_mcs_goto(0)-$mom_prev_mcs_goto(0))/$num_pasos_lin]
	set ylinc [expr ($mom_mcs_goto(1)-$mom_prev_mcs_goto(1))/$num_pasos_lin]
	set zlinc [expr ($mom_mcs_goto(2)-$mom_prev_mcs_goto(2))/$num_pasos_lin]
	
	set ilinc [expr ($mom_tool_axis(0)-$mom_prev_tool_axis(0))/$num_pasos_lin]
	set jlinc [expr ($mom_tool_axis(1)-$mom_prev_tool_axis(1))/$num_pasos_lin]
	set klinc [expr ($mom_tool_axis(2)-$mom_prev_tool_axis(2))/$num_pasos_lin]
	

	set Cabs $Cabs_antiguo
	set inc2 [expr ($Bsave-$Bant)/$num_pasos_lin]
	set Bbase $Bant
	set componente_k $mom_tool_axis(2)
while {$contador < $num_pasos_lin} {
		set contador [expr $contador + 1]
        	set xi [expr $mom_prev_mcs_goto(0) + $xlinc*$contador]
		set yi [expr $mom_prev_mcs_goto(1) + $ylinc*$contador]
		set zi [expr $mom_prev_mcs_goto(2) + $zlinc*$contador]
		#miro si se sale o llega de la vertical o no
		
		if {$mom_prev_tool_axis(2) < 0.9999999999 && $componente_k < 0.99999999} {
			# no viene de la vertical ni va
			set mom_tool_axis(0) [expr $mom_prev_tool_axis(0) + $ilinc*$contador]
			set mom_tool_axis(1) [expr $mom_prev_tool_axis(1) + $jlinc*$contador]
			set mom_tool_axis(2) [expr $mom_prev_tool_axis(2) + $klinc*$contador]
		} else {
			# viene de la vertical o va a la vertical
			set inc3 [expr (($Csave-$Cabs_antiguo)/$num_pasos_lin)*$contador]
			set mom_tool_axis(0) [expr 0.000002*cos(($Cabs_antiguo+$inc3)*$pi/180)]
			set mom_tool_axis(1) [expr 0.000002*sin(($Cabs_antiguo+$inc3)*$pi/180)]
			set mom_tool_axis(2) [expr $mom_prev_tool_axis(2) + $klinc*$contador]
			#MOM_output_literal "angulo [expr $contador*$tollin]"
			#MOM_output_literal "mom_tool_axis(0) $mom_tool_axis(0)"
			#MOM_output_literal "mom_tool_axis(1) $mom_tool_axis(1)"
			#MOM_output_literal "mom_tool_axis(2) $mom_tool_axis(2)"
		} 
		#MOM_output_literal "mom_tool_axis(0) $mom_tool_axis(0)"
		#MOM_output_literal "mom_tool_axis(1) $mom_tool_axis(1)"
		#MOM_output_literal "mom_tool_axis(2) $mom_tool_axis(2)"
        	calcula_ang
		#el angulo B lo calculo otra vez entre 0 y B final si se pasa de hta vertical a otro angulo
		if {$mom_prev_tool_axis(2) > 0.9999999999} { 
			set B [expr $Bbase + $inc2*$contador]
		}
        	transforma_coord
		#MOM_output_literal "C $C Cant $Cant Cabs $Cabs"
		set inc1 [expr $C - $Cant]
	
	#busco el camino ms corto
		if {$inc1 < -180} {
			set Cinc  [expr 360 + $inc1]
		} 
	
		if {$inc1 > 180} {
			set Cinc  [expr $inc1 - 360]
		} 
        
		if {$inc1 > -180 && $inc1 < 180} {
			set Cinc $inc1
		}
	        
	
		set Cabs [expr $Cabs + $Cinc]
		#MOM_output_literal "Cabs $Cabs inc1 $inc1"
		MOM_do_template linear
		VNC_linear_move
		set Cant $C
		set Bant $B
		#VNC_linear
		
}
}
proc calcula_alfa {} {
#_______________________________________________________________________________
# para obtener el angulo conociendo las coordenadas x e y(ij). "A" es el mdulo del vector
#
#_______________________________________________________________________________
global i j
global alfa
global pi
global A
global mom_tool_axis
global tipo_angulo

if {$tipo_angulo == 0} {
	#MOM_output_literal "angulo alfa para angulos"
	set i $mom_tool_axis(0)
	set j $mom_tool_axis(1)
}

if {[expr abs($i)] > 0.0000000001} {
	set alfa [expr atan($j/$i)]
} else {
	set alfa [expr $pi/2]
}
	if {$i > 0 && $j > 0} {
		set alfa $alfa
	} elseif {$i < 0 && $j > 0} {
		set alfa [expr $pi - abs($alfa)]
	} elseif {$i < 0 && $j < 0} {
		set alfa [expr $pi + abs($alfa)]
	} elseif {$i > 0 && $j < 0} {
		set alfa [expr 2*$pi - abs($alfa)]
	} 
	
	#para 0 90 180 o 270
	if {[expr abs($i)] < 0.0000000001 && [expr $j] > 0} {
		set alfa [expr $pi/2]
	}
	if {[expr abs($i)] < 0.0000000001 && [expr $j] < 0} {
		
		set alfa [expr 3*$pi/2]
	}
	if {[expr $i] > 0.0 && [expr abs($j)] < 0.0000000001} {
		set alfa 0
		
	}
	if {[expr $i] < 0.0 && [expr abs($j)] < 0.0000000001} {
		set alfa $pi
		
	}

set A [expr sqrt($i*$i+$j*$j)]
#MOM_output_literal "tipo ang $tipo_angulo alfa $alfa A $A i $i j $j"
}

proc  calcula_ang {} {
#_______________________________________________________________________________
# para obtener el giro B y C para posicionamiento
# alfa es el angulo del vector de hta respecto a X segn el plano Z
# beta es el angulo del vector de hta respecto a la vertical Z
# o x y son variables intermedias
#_______________________________________________________________________________
global mom_tool_axis
global pi
global B
global C
global beta
global alfa
global o x y
global i j
global tipo_angulo
global B_inf B_sup B_ant
global num_iter

set tipo_angulo 0

	set beta [expr $pi/2 - acos($mom_tool_axis(2))]
	#MOM_output_literal "[expr $beta*180/$pi]"

	set o [expr 2*sin($beta)*sin($beta)]
	
	set B_inf 0
	set B_sup [expr $pi]
	set B_ant 100
	set B	0
	set num_iter 0

# iteraciones para buscar B
	while {[expr abs($B_ant - $B)] > 0.0000001} {
		set B_ant $B
		set num_iter [expr $num_iter+1]
		if {$num_iter > 1000000} {
			break
		}
		set B [expr ($B_sup + $B_inf)/2]
		set x [expr cos($B)]
		set y [expr (1+$x*$x)*sin($pi/4+atan(1/$x))*sin($pi/4+atan(1/$x))]
		#MOM_output_literal "B $B B_inf $B_inf B_sup $B_sup B_ant $B_ant x $x y $y"
		if {$y > $o} {
			set B_inf $B
		} else {
			set B_sup $B
		}
		
	}
	#MOM_output_literal "numero de iteraciones $num_iter"

	#esta parte hay que rehacerla para hacerla de forma iterativa
	#buscar entre B0 y B180
	#while {$y > $o} {
	#	set x [expr cos($B)]
	#	set y [expr (1+$x*$x)*sin($pi/4+atan(1/$x))*sin($pi/4+atan(1/$x))]
	#	set B [expr $B+$pi/100000]
	#}
	
	set C [expr sqrt(1+cos($B)*cos($B))]
	set C [expr $C*cos($pi/4+atan(1/cos($B)))]
	set C [expr atan($C/sin($B))]
	set C [expr abs($C)]
	#set C [expr atan(sqrt(1+cos($B)*cos($B))*cos(pi/4+atan(1/cos($B))))/sin($B)])
	#lo anterior es para alfa=0

	calcula_alfa
	# Herramienta vertical
	if {$mom_tool_axis(2) > 0.99999999999} {
		set alfa 0
		set C	 0
		set B    0
	}
	set C [expr $alfa + $C]
	if {$C > 2*$pi} {
		set C [expr $C - 2*$pi]
	}
	set B [expr $B*180/$pi]
	set C [expr abs($C*180/$pi)]
	#MOM_output_literal "B $B C $C"
}


proc transforma_coord {} {
#_______________________________________________________________________________
# transforma las coordenadas xyz a xf yf zf usando angulos B y C
#_______________________________________________________________________________
#
global xi yi zi xf yf zf
global B C
global i j
global pi
global alfa A
global x1 y1 z1
global x2 y2 z2
global x3 y3 z3
global x4 y4 z4
global tipo_angulo

set tipo_angulo 1

	set B [expr $B*$pi/180]
	set C [expr 0-$C*$pi/180]

#Giro C
	set i $xi
	set j $yi
	calcula_alfa
	set x1 [expr $A*cos($alfa+$C)]
	set y1 [expr $A*sin($alfa+$C)]
	set z1 $zi
	#MOM_output_literal "xi $xi yi $yi"
	#MOM_output_literal "alfa $alfa A $A"
	#MOM_output_literal "Giro C x1 $x1 y1 $y1 z1 $z1"

#Giro de 45
	set i $y1
	set j $z1
	calcula_alfa
	set x2 $x1
	set y2 [expr $A*cos($pi/4-$alfa)]
	set z2 [expr 0 - $A*sin($pi/4-$alfa)]
	#MOM_output_literal "alfa $alfa A $A"
	#MOM_output_literal "Giro 45 $x2 $y2 $z2"

#Giro B
	set i $x2
	set j $y2
	calcula_alfa
	set x3 [expr $A*cos($alfa+$B)]
	set y3 [expr $A*sin($alfa+$B)]
	set z3 $z2
	#MOM_output_literal "alfa $alfa A $A"
	#MOM_output_literal "Giro B $x3 $y3 $z3"

#Giro de -45, en sistema de coordenadas originales
	set i $y3
	set j $z3
	calcula_alfa
	set x4 $x3
	set y4 [expr $A*cos($pi/4+$alfa)]
	set z4 [expr $A*sin($pi/4+$alfa)]
	#MOM_output_literal "alfa $alfa A $A"
	#MOM_output_literal "Giro -45 $x4 $y4 $z4"

set xf $x4
set yf $y4
set zf $z4

set B [expr $B*180/$pi]
set C [expr 0-$C*180/$pi]
}




#_______________________________________________________________________________
proc  MOM_start_of_program {} {
#_______________________________________________________________________________
# This procedure is executed at the very begining of the program.
# It gets called before any command is read from the task file.
#_______________________________________________________________________________

#
# initialize simulation system
VNC_start_of_program
         
}


#_______________________________________________________________________________
proc  MOM_start_of_group {} {
#_______________________________________________________________________________
# This procedure is executed at the very begining of the program.
# It gets called before any command is read from the task file.
#_______________________________________________________________________________

global mom_group_name        
	 
	 MOM_set_seq_off
         
         set sufijo "G71"
         MOM_output_literal "%$mom_group_name$sufijo"
	 
         MOM_set_seq_on
         
         MOM_output_literal "G90"

         
}


proc     MOM_end_of_program {} {
#_______________________________________________________________________________
# This procedure is executed at the end of the program after all
# the paths are traversed.
#_______________________________________________________________________________
         global mom_part_name

	 MOM_output_literal "G00 B0 C0"
	 
         MOM_do_template end_of_program
	 
	 

# end of simulation
#VNC_end_of_program


}


proc  MOM_start_of_path {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of each path in the task
# It gets called after all the general parameters of the path are loaded
# like the path name and the tool name.
#_______________________________________________________________________________
         global mom_path_name
         global mom_tool_name
	global mom_fixture_offset_value
	global  C Cabs Cant Cinc B Bant Binc Cabs_antiguo Csave Bsave
	global num_mov
	
	set num_mov 0
   
        
         #MOM_output_literal "(OPERACION: $mom_path_name  HTA: $mom_tool_name)"
         
	#MOM_output_literal "G$mom_fixture_offset_value"

	MOM_force once motion_g X Y Z B C F S
	#MOM_do_template spindle
	
	set a [string range $mom_path_name 0 5]

	if {$a == "G0B0C0"} {
		MOM_output_literal "G0B0C0"
		set Cabs 0
		set Cabs_antiguo 0
		set C 0
		set B 0
		set Cant 0
		set Bant 0
		#set Bsave 0
		#set Csave 0
	}
# ... for simulation
  #VNC_start_of_path
        
}

proc  MOM_end_of_path {} {
#_______________________________________________________________________________
# This procedure is executed at the end of a path.
#_______________________________________________________________________________
         global mom_path_name

          
         #MOM_force once  coolant_m
	 #MOM_do_template coolant_off
         #MOM_output_literal "(FIN OPERACION $mom_path_name)"
# ... simulation
         #VNC_end_of_path
}

proc  MOM_tool_change {} {
#_______________________________________________________________________________
# This procedure is executed at any time there is a tool change.
# It gets called after the tool change data is loaded like the tool number 
# and the register numbers.
#_______________________________________________________________________________
	global mom_path_name
         global mom_tool_name
	global mom_fixture_offset_value
        global num_mov
	global mom_tool_attribute_text

	set num_mov 0 

	#MOM_output_literal "M09"
	#MOM_output_literal "M1"
	MOM_set_seq_off
	MOM_output_literal "; ($mom_tool_name)"	
	MOM_set_seq_on
	
	MOM_do_template tool_change
	#MOM_output_literal "G$mom_fixture_offset_value"
	MOM_force once  spindle_m coolant_m S T H
	#MOM_do_template spindle_start
  
  

  set mom_tool_attribute_text "NONE"

  #VNC_mount_tool $mom_tool_name
  VNC_tool_change

}


proc  MOM_first_tool {} {
#_______________________________________________________________________________
# This procedure is executed for the first operation, with a tool, in the task.
#_______________________________________________________________________________
	global mom_path_name
         global mom_tool_name
	global mom_fixture_offset_value
	global num_mov
	#global mom_tool_attribute_text

	
	set num_mov 0
	#MOM_output_literal "M1"
	MOM_set_seq_off
	MOM_output_literal "; ($mom_tool_name)"
	MOM_set_seq_on
	MOM_force once S spindle_m		
	MOM_do_template first_tool_change
	#MOM_output_literal "G$mom_fixture_offset_value"
	#MOM_force once  spindle_m coolant_m S T H
	#MOM_do_template spindle_start
	
	
	#MOM_output_literal "G90"

 set mom_tool_attribute_text "NONE"
  
  #VNC_mount_tool $mom_tool_name
  VNC_tool_change

}


#_______________________________________________________________________________
proc MOM_msys {} {
#_______________________________________________________________________________
#
# set the current MSYS as a new ZCS. The NC program is defined in
# the ZCS ccordinate system 
  VNC_set_msys_as_zcs
  
  # ESTO ERA ANTES
  # set the current ZCS which is the coordinate system that corresponds
  # to MCS and is used during simulation.
  # VNC_msys "MACHINE_BASE"
}


proc  MOM_linear_move {} {
#_______________________________________________________________________________
# This procedure is executed for each linear move. It gets called after
# the positioning varables, position and tool axis (X,Y,Z,TAX,TAY,TAZ) are loaded.
#_______________________________________________________________________________
	global mom_pos
	global xi yi zi xf yf zf
	global  C Cabs Cant Cinc B Bant Binc
	global inc1
	global num_mov
	global num_pasos_lin
	global tollin
	global xfsave yfsave zfsave Bsave Csave Cfinal
	global Cabs_antiguo
	global mom_feed_rate avance
	global mom_seqnum
	
	if {$mom_seqnum > 90000} {set mom_seqnum 5}

	set num_mov [expr $num_mov+1]

        set xi $mom_pos(0)
	set yi $mom_pos(1)
	set zi $mom_pos(2)
	
        calcula_ang
	set Cfinal $C
        transforma_coord

	set inc1 [expr $C - $Cant]
	
#busco el camino ms corto
	if {$inc1 < -180} {
		set Cinc  [expr 360 + $inc1]
	} 
	
	if {$inc1 > 180} {
		set Cinc  [expr $inc1 - 360]
	} 
        
	if {$inc1 > -180 && $inc1 < 180} {
		set Cinc $inc1
	}
	        
	
	set Cabs [expr $Cabs + $Cinc]
	#MOM_output_literal "C $C Cant $Cant Cinc $Cinc Cabs $Cabs"
	set Binc  [expr $B - $Bant]
	#MOM_output_literal "B $B"
#para linearizacion
	if {[expr abs($Cinc)] > $tollin || [expr abs($Binc)] > $tollin} {
		if {[expr abs($Cinc)] > [expr abs($Binc)]} {
			set num_pasos_lin [expr int(abs($Cinc)/$tollin)]
		} else {
			set num_pasos_lin [expr int(abs($Binc)/$tollin)]
		}
	#MOM_output_literal "(NUMERO PUNTOS EN LINEARIZACION $num_pasos_lin)"
	set xfsave $xf
	set yfsave $yf
	set zfsave $zf
	set Bsave $B
	set Csave $Cabs
	
	linearizacion
	#MOM_output_literal "(FIN DE LINEARIZACION)"
	set xf $xfsave
	set yf $yfsave
	set zf $zfsave
	set B $Bsave
	set Cabs $Csave
	}


	set avance $mom_feed_rate

	MOM_do_template linear
	set Cant $Cfinal
	set Cabs_antiguo $Cabs
	set Bant $B

        VNC_linear_move
	
}



proc  MOM_circular_move {} {
#_______________________________________________________________________________
# This procedure is executed for each circular move. It gets called after
# the circle data are loaded (end point and center).
#_______________________________________________________________________________
	global mom_pos
        global mom_pos_arc_axis
	global mom_pos_arc_center
        global g_plane
	global mom_arc_direction
	global mom_arc_angle
	global mom_arc_radius
	global xi yi zi xf yf zf
	global xc yc zc
	global xfc yfc zfc
	global ccw_cw
	global mom_tool_axis
	global mom_arc_axis
	
        set xi $mom_pos(0)
	set yi $mom_pos(1)
	set zi $mom_pos(2)
	
        calcula_ang
        transforma_coord

	set xfc $xf
	set yfc $yf
	set zfc $zf

	set xi $mom_pos_arc_center(0)
	set yi $mom_pos_arc_center(1)
	set zi $mom_pos_arc_center(2)

	transforma_coord

	set xc $xf
	set yc $yf
	set zc $zf
	
	

	if {$mom_arc_angle*$mom_arc_radius*3.14/180. < 0.03 || $mom_arc_radius > 5000} {
		MOM_do_template linear_c
		VNC_linear_move_c
	} else {
		 MOM_force once  motion_g X Y R
	         MOM_do_template circle
		 VNC_circular_move
	         #MOM_suppress off I J K
	}
}

proc  MOM_rapid_move {} {
#_______________________________________________________________________________
# This procedure is executed for each rapid move. It gets called after
# the positioning varables, position and tool axis (X,Y,Z,TAX,TAY,TAZ) are loaded.
#_______________________________________________________________________________
         global mom_pos
	global xi yi zi xf yf zf
	global  C Cabs Cant Cinc B Bant
	global inc1
	global num_mov
	
	set num_mov [expr $num_mov+1]
	
        set xi $mom_pos(0)
	set yi $mom_pos(1)
	set zi $mom_pos(2)
	
        calcula_ang
        transforma_coord
	
	set inc1 [expr $C - $Cant]
	
#busco el camino ms corto
	if {$inc1 < -180} {
		set Cinc  [expr 360 + $inc1]
	} 
	
	if {$inc1 > 180} {
		set Cinc  [expr $inc1 - 360]
	} 
        
	if {$inc1 > -180 && $inc1 < 180} {
		set Cinc $inc1
	}
	        
	
	set Cabs [expr $Cabs + $Cinc]
	#MOM_output_literal "C $C Cant $Cant Cinc $Cinc Cabs $Cabs"
	set Cant $C
	set Bant $B

        if {$num_mov == 1} {
		MOM_suppress once X Y Z
		MOM_force once B C
        	MOM_do_template rapid
        	VNC_rapid_ab
        	
        	MOM_force once X Y 
        	MOM_suppress once Z B C
        	MOM_do_template rapid
        	VNC_rapid_xy
        	
        	MOM_force once Z 
        	MOM_suppress once X Y B C
        	MOM_do_template rapid
        	MOM_force once Z 
        	MOM_suppress once X Y B C
        	VNC_rapid_z
        } else {
        	
		MOM_do_template rapid
		VNC_rapid_xyzab
	}


}

proc  MOM_first_move {} {
#_______________________________________________________________________________
# Se produce en algunos comienzos de operacin
# llama a mov rapido
#_______________________________________________________________________________

MOM_rapid_move
}

proc  MOM_initial_move {} {
#_______________________________________________________________________________
# This procedure is executed for the initial move of each operation.  It assumes 
# the tool is moving from a safe position at rapid to the start of the operation.
#_______________________________________________________________________________

MOM_rapid_move
}
proc  MOM_from_move {} {
#_______________________________________________________________________________
# Se produce en algunos comienzos de operacin
# 
#_______________________________________________________________________________

MOM_rapid_move
}
proc  MOM_gohome_move {} {
#_______________________________________________________________________________
# Se produce al final de operacin si se llama a GOHOME en mov en vacio
# llama a mov rapido
#_______________________________________________________________________________

MOM_rapid_move
}
proc  MOM_before_output {} {
#_______________________________________________________________________________
# This procedure is executed just before a line is about to be output
# to the file. It loads the line into a variable MOM_o_buffer and then calls
# this procedure. When it returns from this procedure, the variable MOM_o_buffer
# is read and written to the file.
#_______________________________________________________________________________

#########The following procedure invokes the listing file with warnings.

         
}

proc  Alignment {} {
#_______________________________________________________________________________
# This procedure is executed in buffering a value and then putting
# the value where it is desired.
#_______________________________________________________________________________
}

proc  cycle_set {} {
#_______________________________________________________________________________
# This procedure is executed for each new cycle.  This will make sure all   
# non-modal addresses and Cycle parameters are output that need to be output
# at the start of a new cycle.
#_______________________________________________________________________________
         global mom_cycle_retract_mode
         global g_return

         MOM_force once motion_g cycle_x cycle_y cycle_z cycle_j F
	  
	
         if {$mom_cycle_retract_mode == "MANUAL"} {
            set g_return 99
         } elseif {$mom_cycle_retract_mode == "AUTO"} {
            set g_return 98
         } else {
            set g_return 99
         }
} 

proc  MOM_cycle_plane_change {} {
#_______________________________________________________________________________
# This procedure is executed at the end of cycle motion when there is a cycle 
# move occuring at a higher level than the previous position.
#_______________________________________________________________________________
        # MOM_do_template cycle_plane_change
}

proc  MOM_cycle_off {} {
#_______________________________________________________________________________
# This procedure is executed at the end of cycle motion. No hace falta G80
#_______________________________________________________________________________
         #MOM_do_template cycle_off
}

proc  MOM_drill {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill cycle.
#__G81_____________________________________________________________________________
	 cycle_set
	 MOM_do_template cycle_drill
	 
}

proc  MOM_drill_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill cycle.
#__G81_____________________________________________________________________________
	global xi yi zi xf yf zf
	global mom_pos
	
        set xi $mom_pos(0)
	set yi $mom_pos(1)
	set zi $mom_pos(2)
	
        calcula_ang
        transforma_coord 
	 MOM_force once X Y Z
	 MOM_do_template cycle_mov
}



proc  MOM_drill_deep {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a deep cycle. 
#__G83_____________________________________________________________________________
global mom_cycle_delay
global tiempo_espera
	if {[info exists mom_cycle_delay]} {
		
		set tiempo_espera $mom_cycle_delay
	} else {
		set tiempo_espera 0
	} 
	 cycle_set
	 MOM_do_template cycle_drill_deep
	 
}

proc  MOM_drill_deep_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a  deep cycle.
#__G83_____________________________________________________________________________
	global xi yi zi xf yf zf
	global mom_pos
	
        set xi $mom_pos(0)
	set yi $mom_pos(1)
	set zi $mom_pos(2)
	
        calcula_ang
        transforma_coord 
       	 MOM_force once X Y Z
	 MOM_do_template cycle_mov
}



proc  MOM_tap {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a tap cycle.
#__G84_____________________________________________________________________________
	 cycle_set
	 MOM_do_template cycle_tap
}

proc  MOM_tap_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a tap cycle.
#__G84_____________________________________________________________________________
	global xi yi zi xf yf zf
	global mom_pos
	
        set xi $mom_pos(0)
	set yi $mom_pos(1)
	set zi $mom_pos(2)
	
        calcula_ang
        transforma_coord 
       	MOM_force once X Y Z 
	MOM_do_template cycle_mov
}



proc  MOM_load_tool {} {
#_______________________________________________________________________________
# This procedure is executed for the LOAD/TOOL command.  All output for the LOAD
# command will take place with the initial move.
#_______________________________________________________________________________
}

proc  MOM_sequence_number {} {
#_______________________________________________________________________________
# This procedure is executed for the Sequence Number command.
#_______________________________________________________________________________
         global mom_sequence_mode
         global mom_sequence_number
         global mom_sequence_increment
         global mom_sequence_frequency
         global mom_sequence_text
         global mom_warning_info
         if {[isset mom_sequence_mode] == "y"} {
            if {$mom_sequence_mode == "OFF"} {
               MOM_set_seq_off
            } elseif {$mom_sequence_mode == "ON"} {
               MOM_set_seq_on
            } elseif {$mom_sequence_mode == "AUTO"} {
               set mom_warning_info \
                   "SEQNO/AUTO not implemented: Event Name: MOM_sequence_number"
            } elseif {$mom_sequence_mode == "N"} {
               MOM_reset_sequence $mom_sequence_number $mom_sequence_increment \
                                  $mom_sequence_frequency
            } else {
               set mom_warning_info \
                   "MOM_SEQUENCE_MODE UNKNOWN: Event Name: MOM_sequence_number"
               MOM_catch_warning
            }
         } else {
            MOM_reset_sequence $mom_sequence_number $mom_sequence_increment $mom_sequence_frequency
         }
}

proc  MOM_set_modes {} {
#_______________________________________________________________________________
# This procedure is executed for the Set Mode command.
#_______________________________________________________________________________
         global mom_output_mode
         global mom_arc_mode
         global mom_feed_rate_output_mode
         global mom_parallel_to_axis
         global mom_machine_mode
         global mom_modes_text
         global g_output_mode


# "ABSOLUTE", "INCREMENTAL"
            if {$mom_output_mode == "ABSOLUTE"} {
                set g_output_mode "G90"
                MOM_incremental OFF X Y Z
            } elseif {$mom_output_mode == "INCREMENTAL"} {
                set g_output_mode "G91"
                MOM_incremental ON X Y Z
            }
# "LINEAR", "CIRCULAR"
            if {$mom_arc_mode == "LINEAR"} {
            } elseif {$mom_arc_mode == "CIRCULAR"} {
            }
# "OFF", "IPM", "IPR", "MMPM", "MMPR", "INVERSE"
            if {$mom_feed_rate_output_mode == "FPM"} {
            } elseif {$mom_feed_rate_output_mode == "FPR"} {
            } elseif {$mom_feed_rate_output_mode == "INVERSE"} {
            } elseif {$mom_feed_rate_output_mode == "OFF"} {
            }
# "ZAXIS", "WAXIS", "UAXIS"
            if {$mom_parallel_to_axis == "ZAXIS"} {
            } elseif {$mom_parallel_to_axis == "WAXIS"} {
            } elseif {$mom_parallel_to_axis == "UAXIS"} {
            }
# "MILL", "TURN", "PUNCH", "LASER", "TORCH", "WIRE"
            if {$mom_machine_mode == "MILL"} {
            } elseif {$mom_machine_mode == "TURN"} {
            } elseif {$mom_machine_mode == "PUNCH"} {
            } elseif {$mom_machine_mode == "LASER"} {
            } elseif {$mom_machine_mode == "TORCH"} {
            } elseif {$mom_machine_mode == "WIRE"} {
            }
}

proc  MOM_cutcom_on {} {
#_______________________________________________________________________________
# This procedure is executed when the Cutcom command is activated.
#_______________________________________________________________________________




         MOM_do_template cutcom_on
}

proc  MOM_cutcom_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Cutcom command is deactivated.
#_______________________________________________________________________________
global mom_cutcom_status

         #set mom_cutcom_status "OFF"
         MOM_do_template cutcom_off

}

proc  MOM_spindle_rpm {} {
#_______________________________________________________________________________
# This procedure is executed when the Spindle command is activated.
#_______________________________________________________________________________
}

proc  MOM_spindle_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Spindle command is deactivated.
#_______________________________________________________________________________
         MOM_do_template spindle_off
}

proc  MOM_coolant_on {} {
#_______________________________________________________________________________
# This procedure is executed when the Coolant command is activated.
#_______________________________________________________________________________
MOM_do_template coolant_on
}

proc  MOM_coolant_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Coolant command is deactivated.
#_______________________________________________________________________________
         MOM_do_template coolant_off
}

proc  MOM_opstop {} {
#_______________________________________________________________________________
# This procedure is executed when the Optional Stop command is activated.
#_______________________________________________________________________________
         MOM_do_template opstop
}

proc  MOM_stop {} {
#_______________________________________________________________________________
# This procedure is executed when the Program Stop command is activated.
#_______________________________________________________________________________
         MOM_do_template stop
}

proc  MOM_opskip_on {} {MOM_set_line_leader always "/"}
proc  MOM_opskip_off {} {MOM_set_line_leader off "/"}

proc  MOM_delay {} {
#_______________________________________________________________________________
# This procedure is executed when the Delay command is activated.
#_______________________________________________________________________________
         MOM_do_template delay
}

proc  MOM_auxfun {} {
#_______________________________________________________________________________
# This procedure is executed when the Auxiliary command is activated.
#_______________________________________________________________________________
         MOM_do_template auxiliary
}

proc  MOM_prefun {} {
#_______________________________________________________________________________
# This procedure is executed when the Preparatory function command is activated.
#_______________________________________________________________________________
         MOM_do_template preparatory
}

proc  MOM_length_compensation {} {
#_______________________________________________________________________________
# This procedure is executed when the Tool Length Compensation command is activated.
#_______________________________________________________________________________
}

proc  MOM_tool_preselect {} {
#_______________________________________________________________________________
# This procedure is executed when the Tool Preselect command is activated.
#_______________________________________________________________________________
}

proc  MOM_text {} {
#_______________________________________________________________________________
# This procedure is executed when the Text command is activated.
#_______________________________________________________________________________
         global   mom_user_defined_text
         MOM_output_literal "$mom_user_defined_text" 
}

proc  MOM_insert {} {
#_______________________________________________________________________________
# This procedure is executed when the Text command is activated.
#_______________________________________________________________________________
         global   mom_Instruction
         MOM_output_literal "$mom_Instruction" 
}

proc  MOM_pprint {} {
#_______________________________________________________________________________
# This procedure is executed when the Text command is activated.
#_______________________________________________________________________________
         global   mom_pprint
         MOM_output_literal "; ($mom_pprint)" 
}
################################################################################

proc MOM_before_motion {} {
}
proc MOM_machine_mode {} {
}
proc MOM_catch_warning {} {
}